/*******************************************************************************
filename      NamedVariantType.cpp
author        Craig Williams

Brief Description:
  Same as the VariantType, except a additional string can be associated with
    this container. This is very useful for Data Value pairs or similar.

  Name value is store and accessable through the member variable mName.

  See VariantType.h for more information.
*******************************************************************************/
#include <stdio.h>

#include "NamedVariantType.h"

NamedVariantType::NamedVariantType(void                                    ) : VariantType(   ) {             }
NamedVariantType::NamedVariantType(const CString &name, void           *val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, bool           *val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, bool            val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, char           *val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, char            val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, unsigned char  *val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, unsigned char   val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, short          *val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, short           val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, unsigned short *val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, unsigned short  val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, int            *val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, int             val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, unsigned int   *val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, unsigned int    val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, long           *val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, long            val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, unsigned long  *val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, unsigned long   val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, float          *val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, float           val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, double         *val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, double          val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, long double    *val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, long double     val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, const char     *val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, const CString  &val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, CString        *val) : VariantType(val) {mName = name;}
NamedVariantType::NamedVariantType(const CString &name, const VariantType &val):VariantType(val){mName = name;}
NamedVariantType::NamedVariantType(const NamedVariantType &rval            ) : VariantType(rval){mName = rval.mName;}

CVector<unsigned char> NamedVariantType::Pack(void) const
{
  CVector<unsigned char> data;
  return Pack(data);
}

CVector<unsigned char> &NamedVariantType::Pack(CVector<unsigned char> &data) const
{
  data.push_back(VARIANT_TYPE_NAMED_VARIANT);
  VariantType(mName).Pack(data);
  return VariantType::Pack(data);
}

unsigned int NamedVariantType::Unpack(const CVector<unsigned char> &data, unsigned int start)
{
  unsigned char named = 0;

  if(start >= data.size())
    return start;

  named = data[start];

  if(named == VARIANT_TYPE_NAMED_VARIANT)
  {
    VariantType name;
    start = name.Unpack(data, ++start);
    mName = (CString)name;
  }
  else
    mName.clear();
  return VariantType::Unpack(data, start);
}

void NamedVariantType::Print(void) const
{
  printf("%15s: ", mName.c_str());
  VariantType::Print();
}
